@@ -15,11 +15,13 @@
 #  limitations under the License.
 
 from itertools import chain
+
 import re
 import sys
 import time
 import stat
 from calculate.core.server.gen_pid import search_worked_process2
+from calculate.core.setup_package import ChainProgressTemplate
 from calculate.lib.datavars import DataVars
 from calculate.lib.utils.colortext import TextState, get_color_print, \
     convert_console_to_xml
@@ -45,6 +47,7 @@ import os
 from os import path
 from .datavars import BuilderError
 from .emerge_fetch import EmergeFetcher, EmergeFetcherError
+from calculate.lib.utils.grub import GrubCommand
 
 _ = lambda x: x
 from calculate.lib.cl_lang import (setLocalTranslate, getLazyLocalTranslate)
@@ -65,7 +68,7 @@ class Builder(Update):
         Restore = "builder_restore"
         Image = "builder_image"
         Profile = "builder_profile"
-        UpdateMenu = "builder_menu"
+        UpdateMenu = "update_livemenu"
         All = (Prepare, Break, Update, Restore, Image, Profile)
 
     def init(self):
@@ -1128,6 +1131,7 @@ class Builder(Update):
         """
         if not dn:
             raise BuilderError(_("Failed to remount Flash drive"))
+
         p = process('/bin/mount', '-o', 'remount,rw', dn, stderr=STDOUT)
         if p.failed():
             self.printERROR(p.read().strip())
@@ -1196,3 +1200,79 @@ class Builder(Update):
             self.printWARNING(_("Failed to migrate flash linux directory"))
             self.printWARNING(str(e))
         return True
+
+    def create_iso_grub_cfg(self, dn):
+        gc = GrubCommand()
+        content = []
+        for label, iso, splash in self.clVars.ZipVars(
+                'cl_builder_image_label',
+                'cl_builder_image_iso',
+                'cl_builder_image_splash'):
+            entry = (
+                "menuentry '%(label)s' {\n"
+                "\tset isofile=%(iso)s\n"
+                "\tloopback loop $isofile\n"
+                "\tlinux (loop)/boot/vmlinuz root=live "
+                "iso-scan/filename=$isofile quiet %(splash)s\n"
+                "\tinitrd (loop)/boot/initrd\n}\n\n" % {
+                    'label': label,
+                    'iso': gc.get_relpath(iso),
+                    'splash': 'splash=silent,theme:calculate console=tty1'
+                })
+            content.append(entry)
+        fn_grubcfg = path.join(dn, 'grub.cfg')
+        try:
+            with writeFile(fn_grubcfg) as f:
+                f.write(("\n".join(content)).strip()+"\n")
+        except IOError:
+            raise BuilderError(_("Failed to write %s") % fn_grubcfg)
+        return True
+
+    def clear_iso_grub_cfg(self, dn):
+        if dn:
+            cfg_fn = path.join(dn, 'grub.cfg')
+            if path.exists(cfg_fn):
+                try:
+                    os.unlink(cfg_fn)
+                except OSError:
+                    raise BuilderError(_("Failed to remove %s")%cfg_fn)
+        return True
+
+    def setup_package(self, package):
+        """
+        Обновить конфигурационные файлы системы
+        """
+        clVars = DataVars()
+        try:
+            clVars.importData()
+            clVars.flIniFile()
+            clVars.Set("cl_root_path", "/", True)
+
+            clVars.Set("cl_merge_pkg", [package], True)
+            clVars.Set("cl_action", 'merge', True)
+            for copyvar in ("cl_dispatch_conf", "cl_verbose_set",
+                            "cl_template_path_use",
+                            "builder.cl_livemenu_path",
+                            "builder.cl_builder_livemenu_path"):
+                clVars.Set(copyvar, self.clVars.Get(copyvar), force=True)
+            useClt = self.clVars.Get('cl_template_clt_set') in (True, "on")
+
+            # используем объект шаблонов
+            # с clt шаблонами, clt фильтром, без использования postDispatchConf
+            clTempl = ChainProgressTemplate(
+                self.startTask,
+                self.endTask,
+                self.setProgress,
+                clVars, cltObj=useClt,
+                cltFilter=True,
+                printSUCCESS=self.printSUCCESS,
+                printERROR=self.printERROR,
+                printWARNING=self.printWARNING,
+                askConfirm=self.askConfirm,
+                dispatchConf=self.dispatchConf,
+                printWarning=False)
+            clTempl.applyTemplates()
+        finally:
+            clVars.close()
+            self.endTask()
+        return True
