@@ -205,15 +205,15 @@ class Wsdl(WsdlBase):
             # идентификатор метода
             'method_name': Builder.Method.UpdateMenu,
             # категория метода
-            'category': __('Builder'),
+            'category': __('Update'),
             # заголовок метода
-            'title': __("Update Boot Menu"),
+            'title': __("Update Live-Menu"),
             # иконка для графической консоли
             'image': 'format-indent-less-rtl,format-indent-more',
             # метод присутствует в графической консоли
             'gui': True,
             # консольная команда
-            'command': 'cl-builder-menu',
+            'command': 'cl-update-livemenu',
             # права для запуска метода
             'rights': ['build'],
             # объект содержащий модули для действия
@@ -230,8 +230,12 @@ class Wsdl(WsdlBase):
             # описание груп (список лямбда функций)
             'groups': [
                 lambda group: group(_("Update Boot Menu"),
-                                    normal=('cl_builder_flash_dev_path',),
+                                    normal=(
+                                        'cl_builder_livemenu_type',
+                                        'cl_builder_livemenu_path',),
                                     next_label=_("Perform"))],
+            'brief': {'next': __("Perform"),
+                      'name': __("Update Live-Menu")},
         },
         {
             # идентификатор метода
